//
// KeyboardHook.cpp: Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <windows.h>

// Set up shared data section in DLL.
#pragma data_seg("Shared")

HHOOK g_hHook = NULL;
HWND g_hSubclassedForm = NULL;
int	g_iMsgID;

#pragma data_seg()

// Tell linker to make this section shared and read-write.
#pragma comment(linker, "/section:Shared,rws")

// Global (but not shared).
HINSTANCE g_hDLLInst = NULL;
COPYDATASTRUCT g_copydata;

////////////////////////////////////////////////////////////////////////////////////////////////
// Prototypes of exported functions and keyboard procedure.
////////////////////////////////////////////////////////////////////////////////////////////////

int WINAPI SetGlobalKeyboardHook(int hHandle, int iMsgID);
int WINAPI FreeGlobalKeyboardHook();

LRESULT WINAPI KeyboardProc(int nCode, WPARAM wParam, LPARAM lParam);

////////////////////////////////////////////////////////////////////////////////////////////////
// DllMain.
////////////////////////////////////////////////////////////////////////////////////////////////

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	
	// Keep the instance handle for later use.
	g_hDLLInst = hInst;
	
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			
			// Initialization here.
			break;
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Functions for export.
////////////////////////////////////////////////////////////////////////////////////////////////

/*
 * Sets a keyboard hook and returns its handle, or 0 for failure.
 *
 * Input is handle to subclassed form and message IDs to hook.
 *
 * Sets the shared variable 'g_hSubclassedForm' containing handle of
 * subclassed form, the 'g_iMsgID', and the 'g_hHook' variables.
 */
int WINAPI SetGlobalKeyboardHook(int hSubclassedForm, int iMsgID)
{
	
	// If hook already set, do not set it again.
	if (!g_hHook == 0)
		return 0;
	
	g_hHook = SetWindowsHookEx(
		WH_KEYBOARD,
		(HOOKPROC)KeyboardProc,
		g_hDLLInst,
		0
	);
	
	// Save subclassed form handle.
	g_hSubclassedForm = (HWND)hSubclassedForm;
	
	// Save the message ID(s).
	g_iMsgID = iMsgID;
	
	return (int)g_hHook;
}

int WINAPI FreeGlobalKeyboardHook()
{
	HRESULT hr;
	
	// Free hook hh. Return 0 on failure.
	hr = UnhookWindowsHookEx(g_hHook);
	g_hHook = 0;
	
	return hr;
}

////////////////////////////////////////////////////////////////////////////////////////////////

LRESULT WINAPI KeyboardProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	
	// If 'nCode < 0' then do not process the message.
	if (nCode < 0)
		return CallNextHookEx(g_hHook, nCode, wParam, lParam);
	
	// Check for message ID filter.
	if ((g_iMsgID == 0) || (g_iMsgID & wParam))
	{
		
		// Set up data to copy in message.
		g_copydata.dwData = wParam;		// Message ID here.
		g_copydata.lpData = (PVOID)lParam;
		g_copydata.cbData = NULL;
		
		// Send message.
		if (!g_hSubclassedForm == 0)
		{
			SendMessage(
				g_hSubclassedForm,
				WM_COPYDATA,
				NULL,
				(LPARAM)&(g_copydata.dwData)
			);
		}
	}
	
	// Call next hook in chain.
	return CallNextHookEx(g_hHook, nCode, wParam, lParam);
}